@@ -290,8 +290,8 @@ class IB:
         self.disconnectedEvent.emit()
 
     def isConnected(self) -> bool:
-        """Is there is an API connection to TWS or IB gateway?"""
-        return self.client.isConnected()
+        """Is there an API connection to TWS or IB gateway?"""
+        return self.client.isReady()
 
     def _onError(self, reqId, errorCode, errorString, contract):
         if errorCode == 1102:
@@ -1617,49 +1617,50 @@ class IB:
 
     async def connectAsync(
             self, host: str = '127.0.0.1', port: int = 7497,
-            clientId: int = 1, timeout: float = 4, readonly: bool = False,
-            account: str = ''):
-
-        if self.isConnected():
-            self._logger.warn('Already connected')
-            return self
+            clientId: int = 1, timeout: Optional[float] = 4,
+            readonly: bool = False, account: str = ''):
         self.wrapper.clientId = clientId
-
+        timeout = timeout or None
         try:
             # establish API connection
-            await self.client.connectAsync(
-                host, port, clientId, timeout or None)
+            await self.client.connectAsync(host, port, clientId, timeout)
 
             # autobind manual orders
             if clientId == 0:
                 self.reqAutoOpenOrders(True)
 
-            # request completed orders
-            if not readonly and self.client.serverVersion() >= 150:
-                try:
-                    await asyncio.wait_for(
-                        self.reqCompletedOrdersAsync(False), timeout or None)
-                except asyncio.TimeoutError:
-                    self._logger.error('reqCompletedOrders timed out')
-
-            # request updates for the main account
             accounts = self.client.getAccounts()
-            await asyncio.wait_for(
-                asyncio.gather(
-                    self.reqAccountUpdatesAsync(account or accounts[0]),
-                    self.reqPositionsAsync(),
-                    self.reqExecutionsAsync()),
-                timeout or None)
+            if not account and len(accounts) == 1:
+                account = accounts[0]
 
-            # request updates for sub-accounts, if there are not too many
+            # prepare initializing  requests
+            reqs = {}  # name -> request
+            reqs['positions'] = self.reqPositionsAsync()
+            if not readonly and self.client.serverVersion() >= 150:
+                reqs['completed orders'] = self.reqCompletedOrdersAsync(False)
+            if account:
+                reqs['account updates'] =self.reqAccountUpdatesAsync(account)
             if len(accounts) <= self.MaxSyncedSubAccounts:
-                await asyncio.wait_for(
-                    asyncio.gather(
-                        *(self.reqAccountUpdatesMultiAsync(a)
-                            for a in accounts)),
-                    timeout or None)
-            else:
-                self._logger.warning('Not requesting sub-account updates')
+                for acc in accounts:
+                    reqs[f'account updates for {acc}'] = \
+                        self.reqAccountUpdatesMultiAsync(acc)
+
+            # run initializing requests concurrently and log if any times out
+            tasks = [
+                asyncio.wait_for(req, timeout)
+                for req in reqs.values()]
+            resps = await asyncio.gather(*tasks, return_exceptions=True)
+            for name, resp in zip(reqs, resps):
+                if isinstance(resp, asyncio.TimeoutError):
+                    self._logger.error(f'{name} request timed out')
+
+            # the request for executions must come after all orders are in
+            await asyncio.wait_for(self.reqExecutionsAsync(), timeout)
+
+            # final check if socket is still ready
+            if not self.client.isReady():
+                raise ConnectionError(
+                    'Socket connection broken while connecting')
 
             self._logger.info('Synchronization complete')
             self.connectedEvent.emit()
