@@ -293,13 +293,18 @@ def run(*awaitables: Awaitable, timeout: float = None):
         if loop.is_running():
             return
         loop.run_forever()
-        f = asyncio.gather(*asyncio.Task.all_tasks())
-        f.cancel()
         result = None
-        try:
-            loop.run_until_complete(f)
-        except asyncio.CancelledError:
-            pass
+        all_tasks = (
+            asyncio.all_tasks(loop)  # type: ignore
+            if sys.version_info >= (3, 7) else asyncio.Task.all_tasks())
+        if all_tasks:
+            # cancel pending tasks
+            f = asyncio.gather(*all_tasks)
+            f.cancel()
+            try:
+                loop.run_until_complete(f)
+            except asyncio.CancelledError:
+                pass
     else:
         if len(awaitables) == 1:
             future = awaitables[0]
@@ -439,34 +444,8 @@ def patchAsyncio():
 
 
 def startLoop():
-    """
-    Use nested asyncio event loop for Jupyter notebooks.
-
-    This is not needed anymore in Jupyter versions 5 or higher.
-    """
-    def _ipython_loop_asyncio(kernel):
-        """Use asyncio event loop for the given IPython kernel."""
-        loop = asyncio.get_event_loop()
-
-        def kernel_handler():
-            kernel.do_one_iteration()
-            loop.call_later(kernel._poll_interval, kernel_handler)
-
-        loop.call_soon(kernel_handler)
-        try:
-            if not loop.is_running():
-                loop.run_forever()
-        finally:
-            if not loop.is_running():
-                loop.run_until_complete(loop.shutdown_asyncgens())
-                loop.close()
-
+    """Use nested asyncio event loop for Jupyter notebooks."""
     patchAsyncio()
-    loop = asyncio.get_event_loop()
-    if not loop.is_running():
-        from ipykernel.eventloops import register_integration, enable_gui
-        register_integration('asyncio')(_ipython_loop_asyncio)
-        enable_gui('asyncio')
 
 
 def useQt(qtLib: str = 'PyQt5', period: float = 0.01):
@@ -489,6 +468,7 @@ def useQt(qtLib: str = 'PyQt5', period: float = 0.01):
         qloop.exec_()
         timer.stop()
         stack.append((qloop, timer))
+        qApp.processEvents()
 
     if qtLib not in ('PyQt5', 'PySide2'):
         raise RuntimeError(f'Unknown Qt library: {qtLib}')
